% File:        plot_psd_2D.m
% Copyright (c) 2019 by S. Lawrence Marple Jr.

% This script creates both 3-D and 2-D plots of 2-D spectral analysis results.

% The frequency axis extent in each 'psd2D' dimension are appropriately scaled
% spatial frequencies based on the 2-D signal sampling interval Tx along the
% x-axis and sampling interval Ty along the y-axis. If the 2-D signal being 
% analyzed is complex-valued, the x-axis frequency extent is from -1/(2Tx) to
% +1/(2Tx) and the y-axis frequency extent is from -1/(2Ty) to +1/(2Ty). If the
% 2-D signal being analyzed is real-valued, the x-axis frequency extent is from 
% 0 to +1/(2Tx) and the y-axis frequency extent is from 0 to +1/(2Ty). This is
% a quarter plane of the complex-signal full plane. The other three quarter 
% planes are mirror images of this quarter frequency plane.

[num_freqsx,num_freqsy] = size(psd2D);

if any(any(imag(x)~=0))
    disp(' Data is complex-valued.')
    psd_save = psd2D;
    data_type = 1;
else
    disp(' Data is real-valued.')
    psd_save = psd2D(num_freqsx/2+1:num_freqsx,num_freqsy/2+1:num_freqsy);
    data_type = 2;
end

disp(' 2-D spectral analysis will be scaled in logarithmic units (dB).')
dB_range = input(' Logarithmic range in dB that 2-D PSD is to be plotted (typical range is 40 to 80 dB): ');
dB_psd2D = 10*log10( (1/max(max(psd_save)))*abs(psd_save) + 10^(-dB_range/10) );
% Code above prevents taking logarithm of 0 or - numbers, while setting minimum dB plot level

titletext = input(' Enter text for 2-D spectral analysis title (no quotes around text): ','s');

%**************  3-D Plot  *****************************************************
subplot(2,1,1)
if data_type == 1
    mesh((-.5/Tx:1/(Tx*(num_psdx-1)):.5/Tx),(-.5/Ty:1/(Ty*(num_psdy-1)):.5/Ty),dB_psd2D)
else
    mesh((0:1/(Tx*(num_psdx-1)):.5/Tx),(0:1/(Ty*(num_psdy-1)):.5/Ty),dB_psd2D)
end
xlabel({'Spatial Sampling Frequency','(x-axis/column dimension)'})
ylabel({'Spatial Sampling Frequency','(y-axis/row dimension)'})
zlabel('2-D Spectral Intensity (dB)')
title([titletext,' (3-D Plot)']);

%********************************** 2-D plot 
subplot(2,1,2)
if data_type == 1
    pcolor((-.5/Tx:1/(Tx*(num_psdx-1)):.5/Tx),(-.5/Ty:1/(Ty*(num_psdy-1)):.5/Ty),dB_psd2D)
else
    pcolor((0:1/(Tx*(num_psdx-1)):.5/Tx),(0:1/(Ty*(num_psdy-1)):.5/Ty),dB_psd2D)
end
shading interp
xlabel({'Spatial Sampling Frequency','(x-axis/column dimension)'})
ylabel({'Spatial Sampling Frequency','(y-axis/row dimension)'})
title([titletext,' (2-D Plot)']);
colorbar
%